package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.params.AllocationParam;
import com.qmth.sop.business.bean.params.UserArchivesAllocationParam;
import com.qmth.sop.business.bean.result.CrmArchivesAllocationResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationSubTotalResult;
import com.qmth.sop.business.bean.result.UserArchivesResult;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.mapper.TBUserArchivesAllocationMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.base.BaseEntity;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.RoleTypeEnum;
import com.qmth.sop.common.enums.UserArchivesStatusEnum;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 人员分配表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBUserArchivesAllocationServiceImpl extends ServiceImpl<TBUserArchivesAllocationMapper, TBUserArchivesAllocation> implements TBUserArchivesAllocationService {
    @Resource
    private TBServiceService tbServiceService;
    @Resource
    private TBCrmService tbCrmService;
    @Resource
    private SysRoleService sysRoleService;
    @Resource
    private SysUserRoleService sysUserRoleService;
    @Resource
    private TBUserArchivesService tbUserArchivesService;

    @Override
    public IPage<UserArchivesAllocationResult> findCrmAllocationPage(Long serviceUnitId, String province, String city, String area, String customName, Integer gap, Integer pageNumber, Integer pageSize) {
        province = SystemConstant.translateSpecificSign(province);
        city = SystemConstant.translateSpecificSign(city);
        area = SystemConstant.translateSpecificSign(area);
        customName = SystemConstant.translateSpecificSign(customName);
        return this.baseMapper.findCrmAllocationPage(new Page<>(pageNumber, pageSize), serviceUnitId, province, city, area, customName, gap);
    }

    @Override
    public UserArchivesAllocationSubTotalResult findCrmAllocationSubTotal(Long serviceUnitId, String province, String city, String area, String customName, Integer gap) {
        province = SystemConstant.translateSpecificSign(province);
        city = SystemConstant.translateSpecificSign(city);
        area = SystemConstant.translateSpecificSign(area);
        customName = SystemConstant.translateSpecificSign(customName);
        List<UserArchivesAllocationResult> list = this.baseMapper.findCrmAllocationSubTotal(serviceUnitId, province, city, area, customName, gap);
        Integer publishedCrmCount = Math.toIntExact(list.stream().filter(UserArchivesAllocationResult::getPublish).count());
        Integer totalCrmCount = list.size();

        Integer coordinatorQuota = list.stream().mapToInt(UserArchivesAllocationResult::getCoordinatorQuota).sum();
        Integer coordinatorDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getCoordinatorDistributed).sum();

        Integer effectQuota = list.stream().mapToInt(UserArchivesAllocationResult::getEffectQuota).sum();
        Integer effectDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getEffectDistributed).sum();

        Integer assistantQuota = list.stream().mapToInt(UserArchivesAllocationResult::getAssistantQuota).sum();
        Integer assistantDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getAssistantDistributed).sum();

        UserArchivesAllocationSubTotalResult result = new UserArchivesAllocationSubTotalResult();
        result.setPublishedCrmCount(publishedCrmCount);
        result.setTotalCrmCount(totalCrmCount);
        result.setCoordinatorDistributed(coordinatorDistributed);
        result.setCoordinatorQuota(coordinatorQuota);
        result.setEffectDistributed(effectDistributed);
        result.setEffectQuota(effectQuota);
        result.setAssistantDistributed(assistantDistributed);
        result.setAssistantQuota(assistantQuota);
        return result;
    }

    @Transactional
    @Override
    public void editCrmAllocation(UserArchivesAllocationParam userArchivesAllocationParam) {
        Long serviceUnitId = userArchivesAllocationParam.getServiceUnitId();
        TBService tbService = tbServiceService.getById(serviceUnitId);
        if (Objects.isNull(tbService)) {
            throw ExceptionResultEnum.ERROR.exception("服务单元不存在");
        }

        String crmNo = userArchivesAllocationParam.getCrmNo();
        Long regionUserId = userArchivesAllocationParam.getRegionUserId();

        List<TBCrm> tbCrmList = tbCrmService.list(new QueryWrapper<TBCrm>().lambda().eq(TBCrm::getCrmNo, crmNo));
        if (CollectionUtils.isEmpty(tbCrmList)) {
            throw ExceptionResultEnum.ERROR.exception("派单不存在");
        }
        if (tbCrmList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("存在多个相同派单号的派单[" + crmNo + "]");
        }
        TBCrm tbCrm = tbCrmList.get(0);

        List<TBUserArchivesAllocation> dbList = this.list(new QueryWrapper<TBUserArchivesAllocation>()
                .lambda()
                .eq(TBUserArchivesAllocation::getCrmNo, crmNo));

        if (dbList.stream().anyMatch(e -> Objects.nonNull(e.getSopNo()))) {
            throw ExceptionResultEnum.ERROR.exception("已发布的派单不能调配");
        }

        // 新增关系
        List<AllocationParam> allocationParamList = userArchivesAllocationParam.getAllocationParams();


        List<TBUserArchivesAllocation> tbUserArchivesAllocationList = new ArrayList<>();
        // 处理实施工程师或助理工程师
        if (CollectionUtils.isNotEmpty(allocationParamList)) {
            for (AllocationParam allocationParam : allocationParamList) {
                Long roleId = allocationParam.getRoleId();
                List<Long> userIdList = allocationParam.getUserIdList();
                SysRole role = sysRoleService.getById(roleId);
                if (Objects.isNull(role)) {
                    throw ExceptionResultEnum.ERROR.exception("认证角色不存在");
                } else {
                    if (!RoleTypeEnum.EFFECT_ENGINEER.equals(role.getType()) || !RoleTypeEnum.ASSISTANT_ENGINEER.equals(role.getType())) {
                        throw ExceptionResultEnum.ERROR.exception(String.format("认证角色类型为[%s]不属于[%s]或[%s]",
                                role.getType().getDesc(),
                                RoleTypeEnum.EFFECT_ENGINEER.getDesc(),
                                RoleTypeEnum.ASSISTANT_ENGINEER.getDesc()));
                    }
                }

                for (Long userId : userIdList) {
                    List<Long> sysRoleIdList = sysUserRoleService.listRoleByUserId(userId).stream().map(BaseEntity::getId).distinct().collect(Collectors.toList());
                    if (!sysRoleIdList.contains(roleId)) {
                        throw ExceptionResultEnum.ERROR.exception("所选用户不包含认证角色[" + role.getType().getDesc() + "]");
                    }
                    TBUserArchives tbUserArchives = tbUserArchivesService.findByUserId(userId);
                    if (UserArchivesStatusEnum.FREE.equals(tbUserArchives.getStatus())) {
                        throw ExceptionResultEnum.ERROR.exception("不能分配非空闲状态的用户");
                    }
                    tbUserArchives.setStatus(UserArchivesStatusEnum.NON_EXECUTION_SOP);
                    tbUserArchivesService.updateById(tbUserArchives);

                    TBUserArchivesAllocation tbUserArchivesAllocation = new TBUserArchivesAllocation();
                    tbUserArchivesAllocation.setCrmNo(crmNo);
                    tbUserArchivesAllocation.setArchivesId(tbUserArchives.getId());
                    tbUserArchivesAllocation.setServiceId(serviceUnitId);
                    tbUserArchivesAllocation.setUserId(userId);
                    tbUserArchivesAllocation.setRoleId(roleId);
                    tbUserArchivesAllocation.setRegionUserId(regionUserId);
                    tbUserArchivesAllocationList.add(tbUserArchivesAllocation);
                }
            }
        }

        // 更新区域负责人
        if (regionUserId != null && regionUserId > 0) {
            tbCrm.setRegionCoordinatorId(regionUserId);
            tbCrmService.updateById(tbCrm);
        }

        // 如果存在要分配的记录则根据crm单号删除原先数据并新增
        if (CollectionUtils.isNotEmpty(tbUserArchivesAllocationList)) {
            // 新增前删除派单关系
            if (CollectionUtils.isNotEmpty(dbList)) {
                this.removeByIds(dbList);
            }
            // 新增分配关系
            this.saveBatch(tbUserArchivesAllocationList);
        }
    }

    @Override
    public List<CrmArchivesAllocationResult> findAllocationByCrmNo(String crmNo) {
        List<CrmArchivesAllocationResult> list = this.baseMapper.findAllocationByCrmNo(crmNo);
        TBCrm tbCrm = tbCrmService.findByCrmNo(crmNo);
        Long regionCoordinatorId = tbCrm.getRegionCoordinatorId();

        if (regionCoordinatorId != null && regionCoordinatorId > 0) {

            UserArchivesResult userArchivesResult = tbUserArchivesService.findUserArchivesByArchivesIdORUserId(null, regionCoordinatorId);
            CrmArchivesAllocationResult region = new CrmArchivesAllocationResult();
            region.setType(RoleTypeEnum.REGION_COORDINATOR);
            region.setArchivesId(userArchivesResult.getUserArchivesId());
            region.setArchiverName(userArchivesResult.getName());
            region.setSupplierId(userArchivesResult.getSupplierId());
            region.setSupplierName(userArchivesResult.getSupplierName());
            region.setArchivesProvince(userArchivesResult.getProvince());
            region.setArchivesCity(userArchivesResult.getCity());
            region.setUserId(userArchivesResult.getUserId());
            list.add(region);
        }
        return list;
    }

    @Transactional
    @Override
    public void autoEditCrmAllocationBatch(Long serviceUnitId) {
        List<UserArchivesAllocationResult> crmSourceList = this.baseMapper.findCrmAllocationSubTotal(serviceUnitId, null, null, null, null, null);

        // crm按照差额倒序
        crmSourceList = crmSourceList.stream().filter(e -> e.getUnDistributed() > 0)
                .sorted(Comparator.comparing(UserArchivesAllocationResult::getUnDistributed).reversed())
                .collect(Collectors.toList());

        for (UserArchivesAllocationResult crmSource : crmSourceList) {
            String crmNo = crmSource.getCrmNo();
            String city = crmSource.getCity();

            // 区域协调人差额
            Integer coordinatorUnDistributed = crmSource.getCoordinatorUnDistributed();

            // 实施工程师差额
            Integer effectUnDistributed = crmSource.getEffectUnDistributed();

            // 助理工程师差额
            Integer assistantUnDistributed = crmSource.getAssistantUnDistributed();

            // 该派单已经分配的人员名单
            List<CrmArchivesAllocationResult> crmArchivesAllocationResultList = this.findAllocationByCrmNo(crmNo);
            // 区域协调人
            List<CrmArchivesAllocationResult> coordinatorList = crmArchivesAllocationResultList
                    .stream()
                    .filter(e -> RoleTypeEnum.REGION_COORDINATOR.equals(e.getType()))
                    .collect(Collectors.toList());
            // 实施工程师
            List<CrmArchivesAllocationResult> effectList = crmArchivesAllocationResultList
                    .stream()
                    .filter(e -> RoleTypeEnum.EFFECT_ENGINEER.equals(e.getType()))
                    .collect(Collectors.toList());
            // 助理工程师
            List<CrmArchivesAllocationResult> assistantList = crmArchivesAllocationResultList
                    .stream()
                    .filter(e -> RoleTypeEnum.ASSISTANT_ENGINEER.equals(e.getType()))
                    .collect(Collectors.toList());

            // 先分实施工程师
            if (effectUnDistributed > 0) {
                // 如果存在区域协调人，则要分配的实施工程师必须跟区域协调人处于相同的供应商
                if (CollectionUtils.isNotEmpty(coordinatorList)){
                    CrmArchivesAllocationResult coordinator = coordinatorList.get(0);
                    Long supplierId = coordinator.getSupplierId();

                }
            }

        }

    }
}